<?php

namespace Drupal\sitedash\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\sitedash\Service\SiteDashClient;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure SiteDash settings for this site.
 */
class SiteDashSettingsForm extends ConfigFormBase {

  /**
   * The SiteDash client service.
   *
   * @var \Drupal\sitedash\Service\SiteDashClient
   */
  protected $siteDashClient;

  /**
   * Constructs a new SiteDashSettingsForm.
   *
   * @param \Drupal\sitedash\Service\SiteDashClient $sitedash_client
   *   The SiteDash client service.
   */
  public function __construct(SiteDashClient $sitedash_client) {
    $this->siteDashClient = $sitedash_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('sitedash.client')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['sitedash.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sitedash_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('sitedash.settings');

    // Authentication fieldset.
    $form['authentication'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Authentication'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    ];

    // SiteDash API base URL field.
    $form['authentication']['base_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('SiteDash Backend URL'),
      '#description' => $this->t('The SiteDash API backend URL (without trailing path).'),
      '#maxlength' => 256,
      '#required' => TRUE,
      '#default_value' => $config->get('base_url'),
    ];

    // SiteDash Token field (only shown if not already authenticated).
    if (empty($config->get('pubkey'))) {
      $form['authentication']['token'] = [
        '#type' => 'textfield',
        '#title' => $this->t('SiteDash Token'),
        '#description' => $this->t('Enter your SiteDash token here.'),
        '#maxlength' => 256,
        '#required' => TRUE,
        '#default_value' => $config->get('token') ?: '',
      ];
    }

    // Display site information if we have a public key.
    $pubkey = $config->get('pubkey');
    $siteid = $config->get('siteid');

    if (!empty($pubkey)) {
      // Mask the public key for display.
      $masked_pubkey = str_repeat('*', 6) . substr($pubkey, -6);

      $form['authentication']['site_id'] = [
        '#type' => 'item',
        '#title' => $this->t('Site ID'),
        '#markup' => $siteid ?: $this->t('N/A'),
      ];

      $form['authentication']['site_key'] = [
        '#type' => 'item',
        '#title' => $this->t('Site Key'),
        '#markup' => $masked_pubkey ?: $this->t('N/A'),
      ];

      // Add option to override remote post settings.
      $form['remote_post_settings'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Remote Post Settings'),
        '#collapsible' => FALSE,
        '#collapsed' => FALSE,
      ];

      $form['remote_post_settings']['remote_post_override'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Override Remote Post'),
        '#description' => $this->t('Select this option to have SiteDash manage the Audit Export remote post settings.'),
        '#default_value' => $config->get('remote_post_override') ?: FALSE,
      ];
    }

    // Add authentication or reset button.
    if (empty($config->get('pubkey'))) {
      $form['actions']['authenticate'] = [
        '#type' => 'submit',
        '#value' => $this->t('Authenticate'),
        '#button_type' => 'primary',
        '#submit' => ['::authenticateSubmit'],
      ];
    }
    else {
      $form['actions']['reset'] = [
        '#type' => 'submit',
        '#value' => $this->t('Reset Site Key'),
        '#button_type' => 'danger',
        '#submit' => ['::resetSubmit'],
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('sitedash.settings');

    // Save the base url value.
    if ($form_state->hasValue('base_url')) {
      $config->set('base_url', $form_state->getValue('base_url'));
    }

    // Save the remote_post_override setting if it exists.
    if ($form_state->hasValue('remote_post_override')) {
      $config->set('remote_post_override', $form_state->getValue('remote_post_override'));
    }

    // Save the token if it's in the form.
    if ($form_state->hasValue('token')) {
      $config->set('token', $form_state->getValue('token'));
    }

    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Authentication button submit handler.
   */
  public function authenticateSubmit(array &$form, FormStateInterface $form_state) {
    $token = $form_state->getValue('token');
    $base_url = $form_state->getValue('base_url');

    // Save the token and endpoint first.
    $this->config('sitedash.settings')
      ->set('token', $token)
      ->set('base_url', $base_url)
      ->save();

    // Try to authenticate with SiteDash.
    $this->siteDashClient->authenticate($token);
  }

  /**
   * Reset button submit handler.
   */
  public function resetSubmit(array &$form, FormStateInterface $form_state) {
    $this->siteDashClient->resetAuthentication();
  }

}
