<?php

namespace Drupal\sitedash\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Service for communicating with the SiteDash API.
 */
class SiteDashClient {
  use StringTranslationTrait;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new SiteDashClient.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger channel.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(
    ClientInterface $http_client,
    ConfigFactoryInterface $config_factory,
    LoggerChannelInterface $logger,
    MessengerInterface $messenger,
    RequestStack $request_stack,
  ) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->logger = $logger;
    $this->messenger = $messenger;
    $this->requestStack = $request_stack;
  }

  /**
   * Authenticates with the SiteDash service.
   *
   * @param string $token
   *   The SiteDash token.
   *
   * @return bool
   *   TRUE if authentication was successful, FALSE otherwise.
   */
  public function authenticate(string $token): bool {
    // Get settings for SiteDash.
    $config = $this->configFactory->getEditable('sitedash.settings');
    $site_id = $config->get('siteid') ?: '';
    $env_id = $config->get('envid') ?: '';
    $base_url = $config->get('base_url');

    // Ensure base_url is set.
    if (empty($base_url)) {
      $this->messenger->addError($this->t('SiteDash backend URL is not configured.'));
      return FALSE;
    }

    // Construct the endpoint for site initialization.
    $endpoint = rtrim($base_url, '/') . '/site/init';

    // Get site name from system settings.
    $site_name = $this->configFactory->get('system.site')->get('name');

    // Get host name from the server.
    $host_name = $this->requestStack->getCurrentRequest()->getHost();

    // Prepare the request data.
    $data = [
      'sitedashToken' => $token,
      'siteName' => $site_name,
      'hostName' => $host_name,
      'siteId' => $site_id,
      'envId' => $env_id,
    ];

    try {
      // Send the request to SiteDash API.
      $response = $this->httpClient->request('POST', $endpoint, [
        'json' => $data,
        'timeout' => 10,
        'headers' => [
          'Content-Type' => 'application/json',
        ],
      ]);

      // Process the response.
      if ($response->getStatusCode() === 200) {
        $result = json_decode($response->getBody(), TRUE);

        // Save the response data.
        $config
          ->set('pubkey', $result['publicKey'])
          ->set('siteid', $result['siteId'])
          ->set('envid', $result['environmentId'])
          ->save();

        $this->messenger->addStatus($this->t('Successfully authenticated with SiteDash.'));
        return TRUE;
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to authenticate with SiteDash: @message', [
        '@message' => $e->getMessage(),
      ]);
      $this->messenger->addError($this->t('Failed to authenticate with SiteDash: @message', [
        '@message' => $e->getMessage(),
      ]));
    }

    return FALSE;
  }

  /**
   * Resets the SiteDash authentication.
   */
  public function resetAuthentication(): void {
    $config = $this->configFactory->getEditable('sitedash.settings');
    $config->clear('pubkey')->save();

    $this->messenger->addStatus($this->t('SiteDash authentication has been reset.'));
  }

}
